@@ -30,7 +30,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include "photos-collection-manager.h"
 #include "photos-dlna-renderers-manager.h"
 #include "photos-dropdown.h"
 #include "photos-header-bar.h"
@@ -47,6 +46,7 @@
 struct _PhotosMainToolbarPrivate
 {
   GAction *search;
+  GCancellable *loader_cancellable;
   GSimpleAction *gear_menu;
   GtkWidget *coll_back_button;
   GtkWidget *favorite_button;
@@ -56,7 +56,6 @@ struct _PhotosMainToolbarPrivate
   GtkWidget *selection_button;
   GtkWidget *selection_menu;
   GtkWidget *toolbar;
-  PhotosBaseManager *col_mngr;
   PhotosBaseManager *item_mngr;
   PhotosModeController *mode_cntrlr;
   PhotosRemoteDisplayManager *remote_mngr;
@@ -80,12 +79,12 @@ static void
 photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
-  GObject *active_collection;
+  PhotosBaseItem *active_collection;
   PhotosWindowMode window_mode;
   gboolean selection_mode;
   gchar *primary = NULL;
 
-  active_collection = photos_base_manager_get_active_object (priv->col_mngr);
+  active_collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (priv->item_mngr));
   selection_mode = photos_selection_controller_get_selection_mode (priv->sel_cntrlr);
   window_mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
 
@@ -97,7 +96,7 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
       if (!selection_mode)
         {
           if (active_collection != NULL)
-            primary = g_strdup (photos_base_item_get_name (PHOTOS_BASE_ITEM (active_collection)));
+            primary = g_markup_printf_escaped ("%s", photos_base_item_get_name (active_collection));
         }
       else
         {
@@ -115,13 +114,12 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
           if (active_collection != NULL)
             {
               primary =  g_markup_printf_escaped ("<b>%s</b> (%s)",
-                                                  photos_base_item_get_name (PHOTOS_BASE_ITEM (active_collection)),
+                                                  photos_base_item_get_name (active_collection),
                                                   label);
             }
           else
             {
-              primary = label;
-              label = NULL;
+              primary = g_markup_printf_escaped ("%s", label);
             }
 
           g_free (label);
@@ -133,7 +131,7 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
 
       item = photos_base_manager_get_active_object (priv->item_mngr);
       if (item != NULL)
-        primary = g_strdup (photos_base_item_get_name (PHOTOS_BASE_ITEM (item)));
+        primary = g_markup_printf_escaped ("%s", photos_base_item_get_name (PHOTOS_BASE_ITEM (item)));
     }
 
   if (selection_mode)
@@ -176,12 +174,12 @@ photos_main_toolbar_coll_back_button_clicked (PhotosMainToolbar *self)
 
 
 static void
-photos_main_toolbar_col_active_changed (PhotosMainToolbar *self, GObject *object)
+photos_main_toolbar_col_active_changed (PhotosMainToolbar *self, PhotosBaseItem *collection)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
   PhotosHeaderBarMode mode;
 
-  if (object != NULL)
+  if (collection != NULL)
     {
       mode = PHOTOS_HEADER_BAR_MODE_STANDALONE;
       if (priv->coll_back_button == NULL)
@@ -310,8 +308,8 @@ photos_main_toolbar_add_selection_button (PhotosMainToolbar *self)
                             G_CALLBACK (photos_main_toolbar_select_button_clicked),
                             self);
 
-  g_signal_connect_object (priv->col_mngr,
-                           "active-changed",
+  g_signal_connect_object (priv->item_mngr,
+                           "active-collection-changed",
                            G_CALLBACK (photos_main_toolbar_col_active_changed),
                            self,
                            G_CONNECT_SWAPPED);
@@ -325,6 +323,9 @@ photos_main_toolbar_back_button_clicked (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
 
+  if (priv->loader_cancellable != NULL)
+    g_cancellable_cancel (priv->loader_cancellable);
+
   photos_base_manager_set_active_object (priv->item_mngr, NULL);
   photos_mode_controller_go_back (priv->mode_cntrlr);
 }
@@ -364,11 +365,11 @@ photos_main_toolbar_clear_state_data (PhotosMainToolbar *self)
       g_variant_unref (state);
     }
 
-  if (priv->col_mngr != NULL)
-    g_signal_handlers_disconnect_by_func (priv->col_mngr, photos_main_toolbar_col_active_changed, self);
-
   if (priv->item_mngr != NULL)
-    g_signal_handlers_disconnect_by_func (priv->item_mngr, photos_main_toolbar_item_active_changed, self);
+    {
+      g_signal_handlers_disconnect_by_func (priv->item_mngr, photos_main_toolbar_col_active_changed, self);
+      g_signal_handlers_disconnect_by_func (priv->item_mngr, photos_main_toolbar_item_active_changed, self);
+    }
 
   if (priv->sel_cntrlr != NULL)
     g_signal_handlers_disconnect_by_func (priv->sel_cntrlr, photos_main_toolbar_set_toolbar_title, self);
@@ -491,7 +492,7 @@ static void
 photos_main_toolbar_populate_for_collections (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
-  GObject *object;
+  PhotosBaseItem *collection;
 
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
@@ -501,8 +502,8 @@ photos_main_toolbar_populate_for_collections (PhotosMainToolbar *self)
   if (gtk_widget_get_parent (priv->searchbar) == NULL)
     gtk_container_add (GTK_CONTAINER (self), priv->searchbar);
 
-  object = photos_base_manager_get_active_object (priv->col_mngr);
-  photos_main_toolbar_col_active_changed (self, object);
+  collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (priv->item_mngr));
+  photos_main_toolbar_col_active_changed (self, collection);
 }
 
 
@@ -528,7 +529,7 @@ static void
 photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
-  GObject *object;
+  PhotosBaseItem *collection;
 
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
@@ -538,8 +539,8 @@ photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
   if (gtk_widget_get_parent (priv->searchbar) == NULL)
     gtk_container_add (GTK_CONTAINER (self), priv->searchbar);
 
-  object = photos_base_manager_get_active_object (priv->col_mngr);
-  photos_main_toolbar_col_active_changed (self, object);
+  collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (priv->item_mngr));
+  photos_main_toolbar_col_active_changed (self, collection);
 }
 
 
@@ -547,7 +548,7 @@ static void
 photos_main_toolbar_populate_for_overview (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
-  GObject *object;
+  PhotosBaseItem *collection;
 
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
@@ -557,8 +558,8 @@ photos_main_toolbar_populate_for_overview (PhotosMainToolbar *self)
   if (gtk_widget_get_parent (priv->searchbar) == NULL)
     gtk_container_add (GTK_CONTAINER (self), priv->searchbar);
 
-  object = photos_base_manager_get_active_object (priv->col_mngr);
-  photos_main_toolbar_col_active_changed (self, object);
+  collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (priv->item_mngr));
+  photos_main_toolbar_col_active_changed (self, collection);
 }
 
 
@@ -624,7 +625,7 @@ static void
 photos_main_toolbar_populate_for_search (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
-  GObject *object;
+  PhotosBaseItem *collection;
 
   gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
@@ -634,8 +635,8 @@ photos_main_toolbar_populate_for_search (PhotosMainToolbar *self)
   if (gtk_widget_get_parent (priv->searchbar) == NULL)
     gtk_container_add (GTK_CONTAINER (self), priv->searchbar);
 
-  object = photos_base_manager_get_active_object (priv->col_mngr);
-  photos_main_toolbar_col_active_changed (self, object);
+  collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (priv->item_mngr));
+  photos_main_toolbar_col_active_changed (self, collection);
 }
 
 
@@ -690,8 +691,8 @@ photos_main_toolbar_dispose (GObject *object)
 
   photos_main_toolbar_clear_state_data (self);
 
+  g_clear_object (&priv->loader_cancellable);
   g_clear_object (&priv->searchbar);
-  g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->mode_cntrlr);
   g_clear_object (&priv->remote_mngr);
@@ -777,8 +778,7 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
 
   photos_header_bar_set_selection_menu (PHOTOS_HEADER_BAR (priv->toolbar), GTK_BUTTON (priv->selection_menu));
 
-  priv->col_mngr = g_object_ref (state->col_mngr);
-  priv->item_mngr = photos_item_manager_dup_singleton ();
+  priv->item_mngr = g_object_ref (state->item_mngr);
 
   priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
 
@@ -889,6 +889,23 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
 }
 
 
+void
+photos_main_toolbar_set_cancellable (PhotosMainToolbar *self, GCancellable *cancellable)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+
+  if (cancellable == priv->loader_cancellable)
+    return;
+
+  g_clear_object (&priv->loader_cancellable);
+
+  if (cancellable != NULL)
+    g_object_ref (cancellable);
+
+  priv->loader_cancellable = cancellable;
+}
+
+
 void
 photos_main_toolbar_set_stack (PhotosMainToolbar *self, GtkStack *stack)
 {
